#! /usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright 1999-2010 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2010 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


"""
Module that uses :func:`html.astext` to generate a text version of a doc fragment.
Usage: doc2txt spam.xml spam.txt
       to generate spam.txt from spam.xml
"""


__docformat__ = "reStructuredText"


import sys, argparse

from ll.xist import xsc, parse
from ll.xist.ns import html, doc


__docformat__ = "plaintext"


def xsc2txt(infilename, outfilename, title, width):
	e = parse.tree(parse.File(infilename), parse.SGMLOP(), parse.NS(doc), parse.Node(pool=xsc.docpool()))

	if title is None:
		title = xsc.Null
	else:
		title = doc.title(title)
	e = html.html(
		html.body(
			doc.section(title, e)
		)
	)

	e = e.conv()

	with open(outfilename, "wb") as f:
		f.write(html.astext(e, width=width))


def main(args=None):
	p = argparse.ArgumentParser(description="Convert an XML file using the ll.xist.ns.doc namespace into plain text")
	p.add_argument("source", help="input XML file")
	p.add_argument("target", help="output plain text file")
	p.add_argument("-t", "--title", dest="title", help="Title for the document")
	p.add_argument("-w", "--width", dest="width", help="Width of the plain text output (default %(default)s)", type=int, default=72)

	args = p.parse_args()

	xsc2txt(args.source, args.target, args.title, args.width)


if __name__ == "__main__":
	sys.exit(main())



#! /usr/bin/env/python
# -*- coding: utf-8 -*-

## Copyright 1999-2010 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2010 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


import py.test

from ll.xist import xsc
from ll.xist.ns import html, php, chars, abbr


def test_basics():
	# empty pool
	r = xsc.Pool()
	with py.test.raises(xsc.IllegalElementError):
		r.elementclass(u"a", html)
	with py.test.raises(xsc.IllegalElementError):
		r.elementclass_xml(u"a", html)

	# register one element
	r = xsc.Pool(html.a)
	assert r.elementclass(u"a", html) is html.a
	assert r.elementclass_xml(u"a", html) is html.a
	with py.test.raises(xsc.IllegalElementError):
		r.elementclass(u"b", html)
	with py.test.raises(xsc.IllegalElementError):
		r.elementclass_xml(u"b", html)

	# register a module
	r = xsc.Pool(html)
	assert r.elementclass(u"a", html) is html.a
	assert r.elementclass_xml(u"a", html) is html.a
	assert r.elementclass(u"b", html) is html.b
	assert r.elementclass_xml(u"b", html) is html.b
	with py.test.raises(xsc.IllegalElementError):
		r.elementclass(u"c", html)
	with py.test.raises(xsc.IllegalElementError):
		r.elementclass_xml(u"c", html)

	# procinsts
	r = xsc.Pool(php.php)
	assert r.procinstclass(u"php") is php.php
	assert r.procinstclass_xml(u"php") is php.php
	assert r.procinst(u"php", u"foo") == php.php(u"foo")
	assert r.procinst_xml(u"php", u"foo") == php.php(u"foo")
	with py.test.raises(xsc.IllegalProcInstError):
		r.procinstclass(u"nophp")
	with py.test.raises(xsc.IllegalProcInstError):
		r.procinstclass_xml(u"nophp")

	# entities
	r = xsc.Pool(abbr)
	assert r.entityclass(u"xist") is abbr.xist
	assert r.entityclass_xml(u"xist") is abbr.xist
	assert r.entity(u"xist") == abbr.xist()
	assert r.entity_xml(u"xist") == abbr.xist()
	with py.test.raises(xsc.IllegalEntityError):
		r.entityclass(u"dontxist")
	with py.test.raises(xsc.IllegalEntityError):
		r.entityclass_xml(u"dontxist")

	# charrefs
	r = xsc.Pool(chars)
	assert r.charrefclass(u"ouml") is chars.ouml
	assert r.charrefclass_xml(u"ouml") is chars.ouml
	assert r.charrefclass(ord(u"ö")) is chars.ouml
	assert r.charrefclass_xml(ord(u"ö")) is chars.ouml
	assert r.charref(u"ouml") == chars.ouml()
	assert r.charref_xml(u"ouml") == chars.ouml()
	assert r.charref(ord(u"ö")) == chars.ouml()
	assert r.charref_xml(ord(u"ö")) == chars.ouml()
	with py.test.raises(xsc.IllegalEntityError):
		r.charrefclass(u"nothing")
	with py.test.raises(xsc.IllegalEntityError):
		r.charrefclass_xml(u"nothing")


def test_textcomment():
	r = xsc.Pool()
	assert r.text(u"foo") == xsc.Text(u"foo")
	assert r.comment(u"foo") == xsc.Comment(u"foo")


def test_defaultpool():
	r = xsc.threadlocalpool.pool
	assert r.elementclass(u"a", html) is html.a
	assert r.elementclass_xml(u"a", html) is html.a
	assert r.procinstclass(u"php") is php.php
	assert r.procinstclass_xml(u"php") is php.php
	assert r.entityclass(u"xist") is abbr.xist
	assert r.entityclass_xml(u"xist") is abbr.xist
	assert r.charrefclass(u"euro") is chars.euro
	assert r.charrefclass_xml(u"euro") is chars.euro
	assert r.charrefclass(ord(u"\N{EURO SIGN}")) is chars.euro
	assert r.charrefclass_xml(ord(u"\N{EURO SIGN}")) is chars.euro


def test_names():
	# Test classes where the Python and the XML name differ
	with xsc.Pool() as r:
		class element(xsc.Element):
			xmlname = "-element"
			xmlns = "nix"
		class procinst(xsc.ProcInst):
			xmlname = "-procinst"
		class entity(xsc.Entity):
			xmlname = "-entity"
		class charref(xsc.CharRef):
			xmlname = "-charref"
			codepoint = 42
		class Attrs(xsc.Attrs):
			class attr(xsc.TextAttr):
				xmlname = "-attr"
				xmlns = "nix"

	# elements
	assert r.elementclass(u"element", u"nix") is element
	assert r.elementclass_xml(u"-element", u"nix") is element
	assert r.element(u"element", u"nix") == element()
	assert r.element_xml(u"-element", u"nix") == element()
	with py.test.raises(xsc.IllegalElementError):
		r.elementclass(u"-element", u"nix")
	with py.test.raises(xsc.IllegalElementError):
		r.elementclass_xml(u"element", u"nix")
	# make sure that the default pool didn't pick up the new class
	with py.test.raises(xsc.IllegalElementError):
		xsc.threadlocalpool.pool.elementclass(u"element", u"nix")
	with py.test.raises(xsc.IllegalElementError):
		xsc.threadlocalpool.pool.elementclass_xml(u"-element", u"nix")

	# procinsts
	assert r.procinstclass(u"procinst") is procinst
	assert r.procinstclass_xml(u"-procinst") is procinst
	assert r.procinst(u"procinst", u"spam") == procinst(u"spam")
	assert r.procinst_xml(u"-procinst", u"spam") == procinst(u"spam")
	with py.test.raises(xsc.IllegalProcInstError):
		r.procinstclass(u"-procinst")
	with py.test.raises(xsc.IllegalProcInstError):
		r.procinstclass_xml(u"procinst")
	# make sure that the default pool didn't pick up the new class
	with py.test.raises(xsc.IllegalProcInstError):
		xsc.threadlocalpool.pool.procinstclass(u"procinst")
	with py.test.raises(xsc.IllegalProcInstError):
		xsc.threadlocalpool.pool.procinstclass_xml(u"-procinst")

	# entities
	assert r.entityclass(u"entity") is entity
	assert r.entityclass_xml(u"-entity") is entity
	assert r.entity(u"entity") == entity()
	assert r.entity_xml(u"-entity") == entity()
	with py.test.raises(xsc.IllegalEntityError):
		r.entityclass(u"-entity")
	with py.test.raises(xsc.IllegalEntityError):
		r.entityclass_xml(u"entity")
	# make sure that the default pool didn't pick up the new class
	with py.test.raises(xsc.IllegalEntityError):
		xsc.threadlocalpool.pool.entityclass(u"entity")
	with py.test.raises(xsc.IllegalEntityError):
		xsc.threadlocalpool.pool.entityclass_xml(u"-entity")
	# the charref is an entity too
	assert r.entityclass(u"charref") is charref
	assert r.entityclass_xml(u"-charref") is charref
	assert r.entity(u"charref") == charref()
	assert r.entity_xml(u"-charref") == charref()
	with py.test.raises(xsc.IllegalEntityError):
		r.entityclass(u"-charref")
	with py.test.raises(xsc.IllegalEntityError):
		r.entityclass_xml(u"charref")
	# make sure that the default pool didn't pick up the new class
	with py.test.raises(xsc.IllegalEntityError):
		xsc.threadlocalpool.pool.entityclass(u"charref")
	with py.test.raises(xsc.IllegalEntityError):
		xsc.threadlocalpool.pool.entityclass_xml(u"-charref")

	# charrefs
	assert r.charrefclass(u"charref") is charref
	assert r.charrefclass_xml(u"-charref") is charref
	assert r.charrefclass(42) is charref
	assert r.charrefclass_xml(42) is charref
	assert r.charref(u"charref") == charref()
	assert r.charref_xml(u"-charref") == charref()
	assert r.charref(42) == charref()
	assert r.charref_xml(42) == charref()
	with py.test.raises(xsc.IllegalEntityError):
		r.charrefclass(u"-charref")
	with py.test.raises(xsc.IllegalEntityError):
		r.charrefclass_xml(u"charref")
	# make sure that the default pool didn't pick up the new class
	with py.test.raises(xsc.IllegalEntityError):
		xsc.threadlocalpool.pool.charrefclass(u"charref")
	with py.test.raises(xsc.IllegalEntityError):
		xsc.threadlocalpool.pool.charrefclass_xml(u"-charref")
	# make sure that entity has not been register as a charref
	with py.test.raises(xsc.IllegalEntityError):
		xsc.threadlocalpool.pool.charrefclass(u"entity")
	with py.test.raises(xsc.IllegalEntityError):
		xsc.threadlocalpool.pool.charrefclass_xml(u"-entity")

	# attributes
	assert r.attrclass(u"attr", u"nix") is Attrs.attr
	assert r.attrclass_xml(u"-attr", u"nix") is Attrs.attr
	with py.test.raises(xsc.IllegalAttrError):
		r.attrclass(u"-attr", u"nix")
	with py.test.raises(xsc.IllegalAttrError):
		r.attrclass_xml(u"attr", u"nix")
	# make sure that the default pool didn't pick up the new class
	with py.test.raises(xsc.IllegalAttrError):
		xsc.threadlocalpool.pool.attrclass(u"attr", u"nix")
	with py.test.raises(xsc.IllegalAttrError):
		xsc.threadlocalpool.pool.attrclass_xml(u"-attr", u"nix")


def test_names2():
	with xsc.Pool() as r:
		class el_(xsc.Element):
			xmlname = "el"
		class en_(xsc.Entity):
			xmlname = "en"
		class pi_(xsc.ProcInst):
			xmlname = "pi"
		class cr_(xsc.CharRef):
			xmlname = "cr"
			codepoint = 0x4242

	# Test elements
	assert set(r.elements()) == {el_}

	# Test entities
	assert set(r.entities()) == {cr_, en_}

	# Test procinsts
	assert set(r.procinsts()) == {pi_}

	# Test charrefs
	assert set(r.charrefs()) == {cr_}


def test_stack():
	with xsc.Pool() as r1:
		class foo1(xsc.Element):
			xmlname = "foo"
			xmlns = "nix"
		with xsc.Pool() as r2:
			class foo2(xsc.Element):
				xmlname = "foo"
				xmlns = "nix"

	assert r1.elementclass_xml(u"foo", u"nix") is foo1
	assert r2.elementclass_xml(u"foo", u"nix") is foo2


def test_chain():
	with xsc.Pool() as p1:
		class foo1(xsc.Element):
			xmlname = "foo"
			xmlns = "nix"
		class baz(xsc.Element):
			xmlns = "nix"

	with xsc.Pool(p1) as p2:
		class foo2(xsc.Element):
			xmlname = "foo"
			xmlns = "nix"
		class bar(xsc.Element):
			xmlns = "nix"

	assert p1.elementclass_xml(u"foo", u"nix") is foo1
	with py.test.raises(xsc.IllegalElementError):
		p1.elementclass(u"bar", u"nix")
	assert p1.elementclass_xml(u"baz", u"nix") is baz

	assert p2.elementclass_xml(u"foo", u"nix") is foo2
	assert p2.elementclass_xml(u"bar", u"nix") is bar
	assert p2.elementclass_xml(u"baz", u"nix") is baz


def test_chain2():
	with xsc.Pool() as p1:
		class foo1(xsc.Element):
			xmlns = "nix"

	with xsc.Pool() as p2:
		class foo2(xsc.Element):
			xmlns = "nix"

	p = xsc.Pool(p1, p2)
	assert p.elementclass_xml(u"foo2", u"nix") is foo2


def test_mixedattrnames():
	with xsc.Pool() as r:
		class Attrs(xsc.Attrs):
			class a(xsc.TextAttr):
				xmlns = "test"
				xmlname = "A"
			class A(xsc.TextAttr):
				xmlns = "test"
				xmlname = "a"

		class Test(xsc.Element):
			xmlns = "test"
			class Attrs(xsc.Element.Attrs):
				class a(xsc.TextAttr):
					xmlname = "A"
				class A(xsc.TextAttr):
					xmlname = "a"

	xmlns = "test"

	node = Test(
		{
			Attrs.a: "a2",
			Attrs.A: "A2",
		},
		a="a",
		A="A"
	)

	def check(name, value):
		assert unicode(node[name]) == value
		assert unicode(node.attrs[name]) == value
		if isinstance(name, basestring):
			assert unicode(getattr(node.attrs, name)) == value
		assert unicode(node.attrs.get(name)) == value
		if isinstance(name, basestring):
			assert unicode(node.attrs.get_xml(name.swapcase())) == value

	tests = [
		(u"a", u"a"),
		(u"A", u"A"),
		(Test.Attrs.a, u"a"),
		(Test.Attrs.A, u"A"),
		(Attrs.a, u"a2"),
		(Attrs.A, u"A2")
	]
	for (name, value) in tests:
		yield check, name, value

def test_xmlns():
	p = xsc.Pool(html)
	assert p.xmlns == html.xmlns



# -*- coding: utf-8 -*-

## Copyright 1999-2010 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2010 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


"""
A module that allows you to embed tags from Struts__ html custom tag library.

__ http://jakarta.apache.org/struts/
"""


from ll.xist import xsc, sims


__docformat__ = "reStructuredText"


xmlns = "http://jakarta.apache.org/struts/tags-html"


class taglib(xsc.ProcInst):
	"""
	Creates a standard struts taglib header
	"""
	xmlns = xmlns

	def publish(self, publisher):
		yield publisher.encode(u'<%@ taglib uri="/WEB-INF/struts-html.tld" prefix="')
		yield publisher.encode(publisher.getobjectprefix(self))
		yield publisher.encode(u'" %>')


class Element(xsc.Element):
	"""
	Common base class for all the struts html elements
	"""
	register = False


class PartMouseElement(Element):
	xmlns = xmlns
	class Attrs(Element.Attrs):
		class onblur(xsc.TextAttr): pass
		class onchange(xsc.TextAttr): pass
		class onclick(xsc.TextAttr): pass
		class ondblclick(xsc.TextAttr): pass
		class onfocus(xsc.TextAttr): pass
		class onkeydown(xsc.TextAttr): pass
		class onkeypress(xsc.TextAttr): pass
		class onkeyup(xsc.TextAttr): pass
		class onmousedown(xsc.TextAttr): pass
		class onmousemove(xsc.TextAttr): pass
		class onmouseout(xsc.TextAttr): pass
		class onmouseover(xsc.TextAttr): pass
		class onmouseup(xsc.TextAttr): pass
		class style(xsc.TextAttr): pass
		class styleClass(xsc.TextAttr): pass
		class alt(xsc.TextAttr): pass
		class altKey(xsc.TextAttr): pass
		class title(xsc.TextAttr): pass
		class titleKey(xsc.TextAttr): pass
		class value(xsc.TextAttr): pass
		class property(xsc.TextAttr): pass
		class styleId(xsc.TextAttr): pass


class MouseElement(PartMouseElement):
	"""
	Common base class for all the struts elements which have mouse attributes
	"""
	xmlns = xmlns
	class Attrs(PartMouseElement.Attrs):
		class accesskey(xsc.TextAttr): pass
		class tabindex(xsc.TextAttr): pass
		class disabled(xsc.TextAttr): pass


class base(Element):
	"""
	Document base URI
	"""
	xmlns = xmlns
	model = sims.Empty()
	class Attrs(Element.Attrs):
		class target(xsc.URLAttr): pass
		class server(xsc.TextAttr): pass


class button(MouseElement):
	"""
	A button
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(MouseElement.Attrs):
		class indexed(xsc.TextAttr): pass


class cancel(MouseElement):
	"""
	A cancel button
	"""
	xmlns = xmlns
	model = sims.Any()


class checkbox(MouseElement):
	"""
	A html checkbox element
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(MouseElement.Attrs):
		class indexed(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass


class errors(Element):
	"""
	Displays error messages which have been generated from an action or a
	validation method
	"""
	xmlns = xmlns
	model = sims.Empty()
	class Attrs(Element.Attrs):
		class bundle(xsc.TextAttr): pass
		class locale(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class property(xsc.TextAttr): pass


class file(MouseElement):
	"""
	HTML input element of type file
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(MouseElement.Attrs):
		class accept(xsc.TextAttr): pass
		class indexed(xsc.TextAttr): pass
		class maxlength(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class size(xsc.TextAttr): pass


class form(Element):
	"""
	HTML form
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(Element.Attrs):
		class action(xsc.TextAttr): pass
		class enctype(xsc.TextAttr): pass
		class focus(xsc.TextAttr): pass
		class focusIndex(xsc.TextAttr): pass
		class method(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class onreset(xsc.TextAttr): pass
		class onsubmit(xsc.TextAttr): pass
		class scope(xsc.TextAttr): pass
		class style(xsc.TextAttr): pass
		class styleClass(xsc.TextAttr): pass
		class styleId(xsc.TextAttr): pass
		class target(xsc.TextAttr): pass
		class type(xsc.TextAttr): pass


class frame(Element):
	"""
	Render an HTML frame element
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(Element.Attrs):
		class action(xsc.TextAttr): pass
		class anchor(xsc.TextAttr): pass
		class forward(xsc.TextAttr): pass
		class frameborder(xsc.TextAttr): pass
		class frameName(xsc.TextAttr): pass
		class href(xsc.URLAttr): pass
		class longdesc(xsc.TextAttr): pass
		class marginheight(xsc.TextAttr): pass
		class marginwidth(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class noresize(xsc.TextAttr): pass
		class page(xsc.TextAttr): pass
		class paramId(xsc.TextAttr): pass
		class paramName(xsc.TextAttr): pass
		class paramProperty(xsc.TextAttr): pass
		class paramScope(xsc.TextAttr): pass
		class property(xsc.TextAttr): pass
		class scope(xsc.TextAttr): pass
		class scrolling(xsc.TextAttr): pass
		class style(xsc.TextAttr): pass
		class styleClass(xsc.TextAttr): pass
		class styleId(xsc.TextAttr): pass
		class title(xsc.TextAttr): pass
		class titleKey(xsc.TextAttr): pass
		class transaction(xsc.TextAttr): pass


class hidden(PartMouseElement):
	"""
	hidden form field
	"""
	xmlns = xmlns
	model = sims.Empty()
	class Attrs(PartMouseElement.Attrs):
		class accesskey(xsc.TextAttr): pass
		class indexed(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class write(xsc.TextAttr): pass


class html(Element):
	"""
	Render a HTML html element
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(Element.Attrs):
		class locale(xsc.TextAttr): pass
		class xhtml(xsc.TextAttr): pass


class image(MouseElement):
	"""
	image input
	"""
	xmlns = xmlns
	model = sims.Empty()
	class Attrs(MouseElement.Attrs):
		class align(xsc.TextAttr): pass
		class border(xsc.TextAttr): pass
		class bundle(xsc.TextAttr): pass
		class indexed(xsc.TextAttr): pass
		class locale(xsc.TextAttr): pass
		class page(xsc.TextAttr): pass
		class pageKey(xsc.TextAttr): pass
		class src(xsc.TextAttr): pass
		class srcKey(xsc.TextAttr): pass


class img(Element):
	"""
	html img tag
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(Element.Attrs):
		class align(xsc.TextAttr): pass
		class alt(xsc.TextAttr): pass
		class altKey(xsc.TextAttr): pass
		class border(xsc.TextAttr): pass
		class bundle(xsc.TextAttr): pass
		class height(xsc.TextAttr): pass
		class hspace(xsc.TextAttr): pass
		class imageName(xsc.TextAttr): pass
		class ismap(xsc.TextAttr): pass
		class locale(xsc.TextAttr): pass
		class lowsrc(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class onclick(xsc.TextAttr): pass
		class ondblclick(xsc.TextAttr): pass
		class onkeydown(xsc.TextAttr): pass
		class onkeypress(xsc.TextAttr): pass
		class onkeyup(xsc.TextAttr): pass
		class onmousedown(xsc.TextAttr): pass
		class onmousemove(xsc.TextAttr): pass
		class onmouseout(xsc.TextAttr): pass
		class onmouseover(xsc.TextAttr): pass
		class onmouseup(xsc.TextAttr): pass
		class paramId(xsc.TextAttr): pass
		class page(xsc.TextAttr): pass
		class pageKey(xsc.TextAttr): pass
		class paramName(xsc.TextAttr): pass
		class paramProperty(xsc.TextAttr): pass
		class paramScope(xsc.TextAttr): pass
		class property(xsc.TextAttr): pass
		class scope(xsc.TextAttr): pass
		class src(xsc.TextAttr): pass
		class srcKey(xsc.TextAttr): pass
		class style(xsc.TextAttr): pass
		class styleClass(xsc.TextAttr): pass
		class styleId(xsc.TextAttr): pass
		class title(xsc.TextAttr): pass
		class titleKey(xsc.TextAttr): pass
		class usemap(xsc.TextAttr): pass
		class vspace(xsc.TextAttr): pass
		class width(xsc.TextAttr): pass


class javascript(Element):
	"""
	Render JavaScript validation based on the validation rules loaded by the
	ValidatorPlugIn.
	"""
	xmlns = xmlns
	model = sims.Empty()
	class Attrs(Element.Attrs):
		class cdata(xsc.TextAttr): pass
		class dynamicJavascript(xsc.TextAttr): pass
		class formName(xsc.TextAttr): pass
		class method(xsc.TextAttr): pass
		class page(xsc.TextAttr): pass
		class src(xsc.TextAttr): pass
		class staticJavascript(xsc.TextAttr): pass
		class htmlComment(xsc.TextAttr): pass


class link(Element):
	"""
	html link
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(Element.Attrs):
		class accesskey(xsc.TextAttr): pass
		class action(xsc.TextAttr): pass
		class anchor(xsc.TextAttr): pass
		class forward(xsc.TextAttr): pass
		class href(xsc.URLAttr): pass
		class indexed(xsc.TextAttr): pass
		class indexId(xsc.TextAttr): pass
		class linkName(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class onblur(xsc.TextAttr): pass
		class onclick(xsc.TextAttr): pass
		class ondblclick(xsc.TextAttr): pass
		class onfocus(xsc.TextAttr): pass
		class onkeydown(xsc.TextAttr): pass
		class onkeypress(xsc.TextAttr): pass
		class onkeyup(xsc.TextAttr): pass
		class onmousedown(xsc.TextAttr): pass
		class onmousemove(xsc.TextAttr): pass
		class onmouseout(xsc.TextAttr): pass
		class onmouseover(xsc.TextAttr): pass
		class onmouseup(xsc.TextAttr): pass
		class page(xsc.TextAttr): pass
		class paramId(xsc.TextAttr): pass
		class paramName(xsc.TextAttr): pass
		class paramProperty(xsc.TextAttr): pass
		class paramScope(xsc.TextAttr): pass
		class property(xsc.TextAttr): pass
		class scope(xsc.TextAttr): pass
		class style(xsc.TextAttr): pass
		class styleClass(xsc.TextAttr): pass
		class styleId(xsc.TextAttr): pass
		class tabindex(xsc.TextAttr): pass
		class target(xsc.TextAttr): pass
		class title(xsc.TextAttr): pass
		class titleKey(xsc.TextAttr): pass
		class transaction(xsc.TextAttr): pass


class messages(Element):
	"""
	Conditionally display a set of accumulated messages.
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(Element.Attrs):
		class id(xsc.TextAttr): pass
		class bundle(xsc.TextAttr): pass
		class locale(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class property(xsc.TextAttr): pass
		class header(xsc.TextAttr): pass
		class footer(xsc.TextAttr): pass
		class message(xsc.TextAttr): pass


class multibox(MouseElement):
	"""
	multiple checkbox element
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(MouseElement.Attrs):
		class name(xsc.TextAttr): pass


class option(Element):
	"""
	option element
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(Element.Attrs):
		class bundle(xsc.TextAttr): pass
		class dir(xsc.TextAttr): pass
		class disabled(xsc.TextAttr): pass
		class filter(xsc.TextAttr): pass
		class lang(xsc.TextAttr): pass
		class key(xsc.TextAttr): pass
		class locale(xsc.TextAttr): pass
		class style(xsc.TextAttr): pass
		class styleId(xsc.TextAttr): pass
		class styleClass(xsc.TextAttr): pass
		class value(xsc.TextAttr): pass


class options(Element):
	"""
	struts html options element
	"""
	xmlns = xmlns
	model = sims.Empty()
	class Attrs(Element.Attrs):
		class collection(xsc.TextAttr): pass
		class filter(xsc.TextAttr): pass
		class labelName(xsc.TextAttr): pass
		class labelProperty(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class property(xsc.TextAttr): pass
		class style(xsc.TextAttr): pass
		class styleClass(xsc.TextAttr): pass


class optionsCollection(Element):
	"""
	Render a collection of select options
	"""
	xmlns = xmlns
	model = sims.Empty()
	class Attrs(Element.Attrs):
		class filter(xsc.TextAttr): pass
		class label(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class property(xsc.TextAttr): pass
		class style(xsc.TextAttr): pass
		class styleClass(xsc.TextAttr): pass
		class value(xsc.TextAttr): pass


class password(MouseElement):
	"""
	a password text input field
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(MouseElement.Attrs):
		class indexed(xsc.TextAttr): pass
		class maxlength(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class readonly(xsc.TextAttr): pass
		class redisplay(xsc.TextAttr): pass
		class size(xsc.TextAttr): pass


class radio(MouseElement):
	"""
	html input radio
	"""
	xmlns = xmlns
	class Attrs(MouseElement.Attrs):
		class indexed(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class idName(xsc.TextAttr): pass


class reset(MouseElement):
	"""
	a reset button
	"""
	xmlns = xmlns
	model = sims.Any()


class rewrite(Element):
	"""
	render a request uri like html link
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(Element.Attrs):
		class anchor(xsc.TextAttr): pass
		class forward(xsc.TextAttr): pass
		class href(xsc.URLAttr): pass
		class name(xsc.TextAttr): pass
		class page(xsc.TextAttr): pass
		class paramId(xsc.TextAttr): pass
		class paramName(xsc.TextAttr): pass
		class paramProperty(xsc.TextAttr): pass
		class paramScope(xsc.TextAttr): pass
		class property(xsc.TextAttr): pass
		class scope(xsc.TextAttr): pass
		class transaction(xsc.TextAttr): pass


class select(PartMouseElement):
	"""
	a select element text input field
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(PartMouseElement.Attrs):
		class disabled(xsc.TextAttr): pass
		class indexed(xsc.TextAttr): pass
		class multiple(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class tabindex(xsc.TextAttr): pass
		class size(xsc.TextAttr): pass


class submit(MouseElement):
	"""
	a submit button
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(MouseElement.Attrs):
		class indexed(xsc.TextAttr): pass


class text(MouseElement):
	"""
	a text input field
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(MouseElement.Attrs):
		class indexed(xsc.TextAttr): pass
		class maxlength(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class readonly(xsc.TextAttr): pass
		class size(xsc.TextAttr): pass


class textarea(MouseElement):
	"""
	a textarea
	"""
	xmlns = xmlns
	model = sims.Any()
	class Attrs(MouseElement.Attrs):
		class cols(xsc.TextAttr): pass
		class indexed(xsc.TextAttr): pass
		class name(xsc.TextAttr): pass
		class readonly(xsc.TextAttr): pass
		class rows(xsc.TextAttr): pass


class xhtml(Element):
	"""
	Render HTML tags as XHTML
	"""
	xmlns = xmlns
	model = sims.Empty()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright 1999-2010 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2010 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


import sys, argparse, cStringIO

from ll import misc, url
from ll.xist import xsc, xnd, sims


__docformat__ = "reStructuredText"


def iterpath(node):
	yield [node]
	if hasattr(node, "text") and node.text:
		yield [node, node.text]
	if hasattr(node, "getchildren"):
		for child in node:
			for path in iterpath(child):
				yield [node] + path
	if hasattr(node, "tail") and node.tail:
		yield [node, node.tail]


def getelementname(node):
	xmlns = None
	name = node.tag
	if name.startswith("{"):
		(xmlns, sep, name) = name[1:].partition("}")
	return (name, xmlns)


def addetree2xnd(ns, node, elements):
	# Iterate through the tree and collect which elements are encountered and how they are nested
	for path in iterpath(node):
		node = path[-1]
		if "Element" in type(node).__name__:
			(name, xmlns) = getelementname(node)
			if (name, xmlns) in ns.elements:
				xndnode = ns.elements[(name, xmlns)]
			else:
				xndnode = xnd.Element(name, xmlns=xmlns)
				ns += xndnode
				elements[(name, xmlns)] = set()
			for attrname in node.keys():
				if not attrname.startswith("{") and attrname not in xndnode.attrs:
					xndnode += xnd.Attr(attrname, type=xsc.TextAttr)
		elif "ProcessingInstruction" in type(node).__name__:
			name = node.target
			if name not in ns.procinsts:
				ns += xnd.ProcInst(name)
		elif "Comment" in type(node).__name__:
			xndnode = "#comment"
		elif isinstance(node, basestring):
			if node.isspace():
				xndnode = "#whitespace"
			else:
				xndnode = "#text"
		if len(path) >= 2:
			parent = path[-2]
			if "Element" in type(parent).__name__:
				parententry = elements[getelementname(parent)]
				parententry.add(xndnode)


def makexnd(urls, parser="etree", shareattrs="dupes", model="simple", defaultxmlns=None):
	elements = {} # maps (name, xmlns) to content set
	ns = xnd.Module(defaultxmlns=defaultxmlns)
	with url.Context():
		for u in urls:
			if isinstance(u, url.URL):
				u = u.openread()
			elif isinstance(u, str):
				u = cStringIO.StringIO(u)
			if parser == "etree":
				from xml.etree import cElementTree
				node = cElementTree.parse(u).getroot()
			elif parser == "lxml":
				from lxml import etree
				node = etree.parse(u).getroot()
			else:
				raise ValueError("unknown parser {!r}".format(parser))
			addetree2xnd(ns, node, elements)

	# Put sims info into the element definitions
	if model == "none":
		pass
	elif model == "simple":
		for (fullname, modelset) in elements.iteritems():
			ns.elements[fullname].modeltype = bool(modelset)
	elif model in ("fullall", "fullonce"):
		for (fullname, modelset) in elements.iteritems():
			element = ns.elements[fullname]
			if not modelset:
				element.modeltype = "sims.Empty"
			else:
				elements = [el for el in modelset if isinstance(el, xnd.Element)]
				if not elements:
					if "#text" in modelset:
						element.modeltype = "sims.NoElements"
					else:
						element.modeltype = "sims.NoElementsOrText"
				else:
					if "#text" in modelset:
						element.modeltype = "sims.ElementsOrText"
					else:
						element.modeltype = "sims.Elements"
					element.modelargs = elements
	else:
		raise ValueError("unknown sims mode {!r}".format(model))

	if shareattrs=="dupes":
		ns.shareattrs(False)
	elif shareattrs=="all":
		ns.shareattrs(True)
	return ns


def main(args=None):
	p = argparse.ArgumentParser(description="Convert XML files to XIST namespace (on stdout)")
	p.add_argument("urls", metavar="urls", type=url.URL, help="ULRs of DTDs to be parsed", nargs="+")
	p.add_argument("-p", "--parser", dest="parser", help="parser module to use for XML parsing (default: %(default)s)", choices=("etree", "lxml"), default="etree")
	p.add_argument("-s", "--shareattrs", dest="shareattrs", help="Should identical attributes be shared among elements? (default: %(default)s)", choices=("none", "dupes", "all"), default="dupes")
	p.add_argument("-m", "--model", dest="model", help="Create sims info? (default: %(default)s)", choices=("none", "simple", "fullall", "fullonce"), default="simple")
	p.add_argument("-x", "--defaultxmlns", dest="defaultxmlns", metavar="NAME", help="Force elements without a namespace into this namespace")

	args = p.parse_args(args)
	print makexnd(**args.__dict__)


if __name__ == "__main__":
	sys.exit(main())



#! /usr/bin/env/python
# -*- coding: utf-8 -*-

## Copyright 1999-2010 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2010 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


from ll.xist import xsc, sims
from ll.xist.ns import html, php


with xsc.Pool():
	class el1(xsc.Element):
		model = sims.Empty()


# The following tests are split into separate test functions, because ``py.test`` has problems otherwise
def test_empty1():
	e = el1()
	e.bytes()


def test_empty2(recwarn):
	e = el1(u"gurk")
	e.bytes()
	w = recwarn.pop(sims.EmptyElementWithContentWarning)


def test_empty3(recwarn):
	e = el1(php.php(u"gurk"))
	e.bytes()
	w = recwarn.pop(sims.EmptyElementWithContentWarning)


def test_empty4(recwarn):
	e = el1(xsc.Comment(u"gurk"))
	e.bytes()
	w = recwarn.pop(sims.EmptyElementWithContentWarning)


def test_empty5(recwarn):
	e = el1(el1())
	e.bytes()
	w = recwarn.pop(sims.EmptyElementWithContentWarning)


def test_elements(recwarn):
	with xsc.Pool():
		class el11(xsc.Element):
			xmlname = "el1"
			xmlns = "ns1"
		class el12(xsc.Element):
			xmlname = "el2"
			xmlns = "ns1"
		class el21(xsc.Element):
			xmlname = "el1"
			xmlns = "ns2"
		class el22(xsc.Element):
			xmlname = "el2"
			xmlns = "ns2"

		el11.model = sims.Elements(el11, el21)

		e = el11()
		e.bytes()

		e = el11(u"foo")
		e.bytes()
		w = recwarn.pop(sims.IllegalTextWarning)

		e = el11(php.php(u"gurk"))
		e.bytes()

		e = el11(xsc.Comment(u"gurk"))
		e.bytes()

		e = el11(el11())
		e.bytes()

		e = el11(el21())
		e.bytes()

		e = el11(el12())
		e.bytes()
		w = recwarn.pop(sims.WrongElementWarning)

		e = el11(el22())
		e.bytes()
		w = recwarn.pop(sims.WrongElementWarning)


def test_elementsortext(recwarn):
	with xsc.Pool():
		class el11(xsc.Element):
			xmlname = "el1"
			xmlns = "ns1"
		class el12(xsc.Element):
			xmlname = "el2"
			xmlns = "ns1"
		class el21(xsc.Element):
			xmlname = "el1"
			xmlns = "ns2"
		class el22(xsc.Element):
			xmlname = "el2"
			xmlns = "ns2"

		el11.model = sims.ElementsOrText(el11, el21)

		e = el11()
		e.bytes()

		e = el11(u"foo")
		e.bytes()

		e = el11(php.php(u"gurk"))
		e.bytes()

		e = el11(xsc.Comment(u"gurk"))
		e.bytes()

		e = el11(el11())
		e.bytes()

		e = el11(el21())
		e.bytes()

		e = el11(el12())
		e.bytes()
		w = recwarn.pop(sims.WrongElementWarning)

		e = el11(el22())
		e.bytes()
		w = recwarn.pop(sims.WrongElementWarning)


def test_noelements(recwarn):
	with xsc.Pool():
		class el1(xsc.Element):
			xmlns = "ns1"
			model = sims.NoElements()
		class el2(xsc.Element):
			xmlns = "ns2"

		e = el1()
		e.bytes()

		e = el1(u"foo")
		e.bytes()

		e = el1(php.php(u"gurk"))
		e.bytes()

		e = el1(xsc.Comment(u"gurk"))
		e.bytes()

		e = el1(el1())
		e.bytes()
		w = recwarn.pop(sims.ElementWarning)

		# Elements from a different namespace are OK
		e = el1(el2())
		e.bytes()


def test_noelementsortext(recwarn):
	with xsc.Pool():
		class el1(xsc.Element):
			xmlns = "ns1"
			model = sims.NoElementsOrText()
		class el2(xsc.Element):
			xmlns = "ns2"

		e = el1()
		e.bytes()

		e = el1(u"foo")
		e.bytes()
		w = recwarn.pop(sims.IllegalTextWarning)

		e = el1(php.php(u"gurk"))
		e.bytes()

		e = el1(xsc.Comment(u"gurk"))
		e.bytes()

		e = el1(el1())
		e.bytes()
		w = recwarn.pop(sims.ElementWarning)

		# Elements from a different namespace are OK
		e = el1(el2())
		e.bytes()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright 1999-2010 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2010 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


"""
Module that helps to create XIST namespace modules from TLD files (Java tag
library descriptors).

For usage information type::

	$ tld2xsc --help
"""

__docformat__ = "reStructuredText"


import sys, argparse

from ll import misc, url
from ll.xist import xsc, xfind, parse
from ll.xist.ns import tld


__docformat__ = "reStructuredText"


def makexnd(stream, encoding=None, shareattrs="dupes", model="simple"):
	# :var:`stream` can be a stream, an :class:`URL` or a string
	node = parse.tree(stream, parse.Expat(), parse.NS(tld), parse.Node())

	# get and convert the taglib object
	xnd = node.walknodes(tld.taglib)[0].asxnd(model=model)

	if shareattrs=="dupes":
		xnd.shareattrs(False)
	elif shareattrs=="all":
		xnd.shareattrs(True)
	return xnd


def main(args=None):
	p = argparse.ArgumentParser(description="Convert JSP Tag Library Descriptor XML file (on stdin) to XIST namespace (on stdout)")
	p.add_argument("-s", "--shareattrs", dest="shareattrs", help="Should identical attributes be shared among elements? (default %(default)s)", choices=("none", "dupes", "all"), default="dupes")
	p.add_argument("-m", "--model", dest="model", help="Add sims information to the namespace (default %(default)s)", choices=("none", "simple", "fullall", "fullonce"), default="simple")
	p.add_argument("-d", "--defaults", dest="defaults", help="Output default values for attributes? (default %(default)s)", action=misc.FlagAction, default=False)

	args = p.parse_args(args)
	print makexnd(sys.stdin, args.shareattrs, model=args.model, defaults=args.defaults)


if __name__ == "__main__":
	sys.exit(main())
